@@ -24,24 +24,23 @@ class CacheTokenParser extends AbstractTokenParser
     public function parse(Token $token): Node
     {
         $stream = $this->parser->getStream();
-        $expressionParser = $this->parser->getExpressionParser();
-        $key = $expressionParser->parseExpression();
+        $key = $this->parser->parseExpression();
 
         $ttl = null;
         $tags = null;
         while ($stream->test(Token::NAME_TYPE)) {
             $k = $stream->getCurrent()->getValue();
-            if (!in_array($k, ['ttl', 'tags'])) {
+            if (!\in_array($k, ['ttl', 'tags'], true)) {
                 throw new SyntaxError(\sprintf('Unknown "%s" configuration.', $k), $stream->getCurrent()->getLine(), $stream->getSourceContext());
             }
 
             $stream->next();
-            $stream->expect(Token::PUNCTUATION_TYPE, '(');
+            $stream->expect(Token::OPERATOR_TYPE, '(');
             $line = $stream->getCurrent()->getLine();
             if ($stream->test(Token::PUNCTUATION_TYPE, ')')) {
                 throw new SyntaxError(\sprintf('The "%s" modifier takes exactly one argument (0 given).', $k), $line, $stream->getSourceContext());
             }
-            $arg = $expressionParser->parseExpression();
+            $arg = $this->parser->parseExpression();
             if ($stream->test(Token::PUNCTUATION_TYPE, ',')) {
                 throw new SyntaxError(\sprintf('The "%s" modifier takes exactly one argument (2 given).', $k), $line, $stream->getSourceContext());
             }
