@@ -16,7 +16,20 @@ use Twig\Environment;
 use Twig\Error\LoaderError;
 use Twig\Error\RuntimeError;
 use Twig\Error\SyntaxError;
-use Twig\ExpressionParser;
+use Twig\ExpressionParser\Infix\ArrowExpressionParser;
+use Twig\ExpressionParser\Infix\BinaryOperatorExpressionParser;
+use Twig\ExpressionParser\Infix\ConditionalTernaryExpressionParser;
+use Twig\ExpressionParser\Infix\DotExpressionParser;
+use Twig\ExpressionParser\Infix\FilterExpressionParser;
+use Twig\ExpressionParser\Infix\FunctionExpressionParser;
+use Twig\ExpressionParser\Infix\IsExpressionParser;
+use Twig\ExpressionParser\Infix\IsNotExpressionParser;
+use Twig\ExpressionParser\Infix\SquareBracketExpressionParser;
+use Twig\ExpressionParser\InfixAssociativity;
+use Twig\ExpressionParser\PrecedenceChange;
+use Twig\ExpressionParser\Prefix\GroupingExpressionParser;
+use Twig\ExpressionParser\Prefix\LiteralExpressionParser;
+use Twig\ExpressionParser\Prefix\UnaryOperatorExpressionParser;
 use Twig\Markup;
 use Twig\Node\Expression\AbstractExpression;
 use Twig\Node\Expression\Binary\AddBinary;
@@ -63,11 +76,12 @@ use Twig\Node\Expression\Test\EvenTest;
 use Twig\Node\Expression\Test\NullTest;
 use Twig\Node\Expression\Test\OddTest;
 use Twig\Node\Expression\Test\SameasTest;
+use Twig\Node\Expression\Test\TrueTest;
 use Twig\Node\Expression\Unary\NegUnary;
 use Twig\Node\Expression\Unary\NotUnary;
 use Twig\Node\Expression\Unary\PosUnary;
+use Twig\Node\Expression\Unary\SpreadUnary;
 use Twig\Node\Node;
-use Twig\OperatorPrecedenceChange;
 use Twig\Parser;
 use Twig\Sandbox\SecurityNotAllowedMethodError;
 use Twig\Sandbox\SecurityNotAllowedPropertyError;
@@ -305,6 +319,7 @@ final class CoreExtension extends AbstractExtension
             new TwigTest('iterable', 'is_iterable'),
             new TwigTest('sequence', [self::class, 'testSequence']),
             new TwigTest('mapping', [self::class, 'testMapping']),
+            new TwigTest('true', null, ['node_class' => TrueTest::class]),
         ];
     }
 
@@ -313,50 +328,69 @@ final class CoreExtension extends AbstractExtension
         return [];
     }
 
-    public function getOperators(): array
+    public function getExpressionParsers(): array
     {
         return [
-            [
-                'not' => ['precedence' => 50, 'precedence_change' => new OperatorPrecedenceChange('twig/twig', '3.15', 70), 'class' => NotUnary::class],
-                '-' => ['precedence' => 500, 'class' => NegUnary::class],
-                '+' => ['precedence' => 500, 'class' => PosUnary::class],
-            ],
-            [
-                '? :' => ['precedence' => 5, 'class' => ElvisBinary::class, 'associativity' => ExpressionParser::OPERATOR_RIGHT],
-                '?:' => ['precedence' => 5, 'class' => ElvisBinary::class, 'associativity' => ExpressionParser::OPERATOR_RIGHT],
-                '??' => ['precedence' => 300, 'precedence_change' => new OperatorPrecedenceChange('twig/twig', '3.15', 5), 'class' => NullCoalesceBinary::class, 'associativity' => ExpressionParser::OPERATOR_RIGHT],
-                'or' => ['precedence' => 10, 'class' => OrBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'xor' => ['precedence' => 12, 'class' => XorBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'and' => ['precedence' => 15, 'class' => AndBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'b-or' => ['precedence' => 16, 'class' => BitwiseOrBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'b-xor' => ['precedence' => 17, 'class' => BitwiseXorBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'b-and' => ['precedence' => 18, 'class' => BitwiseAndBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '==' => ['precedence' => 20, 'class' => EqualBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '!=' => ['precedence' => 20, 'class' => NotEqualBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '<=>' => ['precedence' => 20, 'class' => SpaceshipBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '<' => ['precedence' => 20, 'class' => LessBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '>' => ['precedence' => 20, 'class' => GreaterBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '>=' => ['precedence' => 20, 'class' => GreaterEqualBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '<=' => ['precedence' => 20, 'class' => LessEqualBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'not in' => ['precedence' => 20, 'class' => NotInBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'in' => ['precedence' => 20, 'class' => InBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'matches' => ['precedence' => 20, 'class' => MatchesBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'starts with' => ['precedence' => 20, 'class' => StartsWithBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'ends with' => ['precedence' => 20, 'class' => EndsWithBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'has some' => ['precedence' => 20, 'class' => HasSomeBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'has every' => ['precedence' => 20, 'class' => HasEveryBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '..' => ['precedence' => 25, 'class' => RangeBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '+' => ['precedence' => 30, 'class' => AddBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '-' => ['precedence' => 30, 'class' => SubBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '~' => ['precedence' => 40, 'precedence_change' => new OperatorPrecedenceChange('twig/twig', '3.15', 27), 'class' => ConcatBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '*' => ['precedence' => 60, 'class' => MulBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '/' => ['precedence' => 60, 'class' => DivBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '//' => ['precedence' => 60, 'class' => FloorDivBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '%' => ['precedence' => 60, 'class' => ModBinary::class, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'is' => ['precedence' => 100, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                'is not' => ['precedence' => 100, 'associativity' => ExpressionParser::OPERATOR_LEFT],
-                '**' => ['precedence' => 200, 'class' => PowerBinary::class, 'associativity' => ExpressionParser::OPERATOR_RIGHT],
-            ],
+            // unary operators
+            new UnaryOperatorExpressionParser(NotUnary::class, 'not', 50, new PrecedenceChange('twig/twig', '3.15', 70)),
+            new UnaryOperatorExpressionParser(SpreadUnary::class, '...', 512, description: 'Spread operator'),
+            new UnaryOperatorExpressionParser(NegUnary::class, '-', 500),
+            new UnaryOperatorExpressionParser(PosUnary::class, '+', 500),
+
+            // binary operators
+            new BinaryOperatorExpressionParser(ElvisBinary::class, '?:', 5, InfixAssociativity::Right, description: 'Elvis operator (a ?: b)', aliases: ['? :']),
+            new BinaryOperatorExpressionParser(NullCoalesceBinary::class, '??', 300, InfixAssociativity::Right, new PrecedenceChange('twig/twig', '3.15', 5), description: 'Null coalescing operator (a ?? b)'),
+            new BinaryOperatorExpressionParser(OrBinary::class, 'or', 10),
+            new BinaryOperatorExpressionParser(XorBinary::class, 'xor', 12),
+            new BinaryOperatorExpressionParser(AndBinary::class, 'and', 15),
+            new BinaryOperatorExpressionParser(BitwiseOrBinary::class, 'b-or', 16),
+            new BinaryOperatorExpressionParser(BitwiseXorBinary::class, 'b-xor', 17),
+            new BinaryOperatorExpressionParser(BitwiseAndBinary::class, 'b-and', 18),
+            new BinaryOperatorExpressionParser(EqualBinary::class, '==', 20),
+            new BinaryOperatorExpressionParser(NotEqualBinary::class, '!=', 20),
+            new BinaryOperatorExpressionParser(SpaceshipBinary::class, '<=>', 20),
+            new BinaryOperatorExpressionParser(LessBinary::class, '<', 20),
+            new BinaryOperatorExpressionParser(GreaterBinary::class, '>', 20),
+            new BinaryOperatorExpressionParser(GreaterEqualBinary::class, '>=', 20),
+            new BinaryOperatorExpressionParser(LessEqualBinary::class, '<=', 20),
+            new BinaryOperatorExpressionParser(NotInBinary::class, 'not in', 20),
+            new BinaryOperatorExpressionParser(InBinary::class, 'in', 20),
+            new BinaryOperatorExpressionParser(MatchesBinary::class, 'matches', 20),
+            new BinaryOperatorExpressionParser(StartsWithBinary::class, 'starts with', 20),
+            new BinaryOperatorExpressionParser(EndsWithBinary::class, 'ends with', 20),
+            new BinaryOperatorExpressionParser(HasSomeBinary::class, 'has some', 20),
+            new BinaryOperatorExpressionParser(HasEveryBinary::class, 'has every', 20),
+            new BinaryOperatorExpressionParser(RangeBinary::class, '..', 25),
+            new BinaryOperatorExpressionParser(AddBinary::class, '+', 30),
+            new BinaryOperatorExpressionParser(SubBinary::class, '-', 30),
+            new BinaryOperatorExpressionParser(ConcatBinary::class, '~', 40, precedenceChange: new PrecedenceChange('twig/twig', '3.15', 27)),
+            new BinaryOperatorExpressionParser(MulBinary::class, '*', 60),
+            new BinaryOperatorExpressionParser(DivBinary::class, '/', 60),
+            new BinaryOperatorExpressionParser(FloorDivBinary::class, '//', 60, description: 'Floor division'),
+            new BinaryOperatorExpressionParser(ModBinary::class, '%', 60),
+            new BinaryOperatorExpressionParser(PowerBinary::class, '**', 200, InfixAssociativity::Right, description: 'Exponentiation operator'),
+
+            // ternary operator
+            new ConditionalTernaryExpressionParser(),
+
+            // Twig callables
+            new IsExpressionParser(),
+            new IsNotExpressionParser(),
+            new FilterExpressionParser(),
+            new FunctionExpressionParser(),
+
+            // get attribute operators
+            new DotExpressionParser(),
+            new SquareBracketExpressionParser(),
+
+            // group expression
+            new GroupingExpressionParser(),
+
+            // arrow function
+            new ArrowExpressionParser(),
+
+            // all literals
+            new LiteralExpressionParser(),
         ];
     }
 
@@ -510,7 +544,6 @@ final class CoreExtension extends AbstractExtension
      * Returns a formatted string.
      *
      * @param string|null $format
-     * @param ...$values
      *
      * @internal
      */
@@ -969,8 +1002,6 @@ final class CoreExtension extends AbstractExtension
      *
      * @param array|\Traversable|string|null $item
      *
-     * @return mixed
-     *
      * @internal
      */
     public static function shuffle(string $charset, $item)
@@ -1405,8 +1436,6 @@ final class CoreExtension extends AbstractExtension
      *        {# ... #}
      *    {% endif %}
      *
-     * @param mixed $value
-     *
      * @internal
      */
     public static function testSequence($value): bool
@@ -1430,8 +1459,6 @@ final class CoreExtension extends AbstractExtension
      *        {# ... #}
      *    {% endif %}
      *
-     * @param mixed $value
-     *
      * @internal
      */
     public static function testMapping($value): bool
@@ -1581,10 +1608,10 @@ final class CoreExtension extends AbstractExtension
     {
         if (null !== $object) {
             if ('class' === $constant) {
-                return $checkDefined ? true : \get_class($object);
+                return $checkDefined ? true : $object::class;
             }
 
-            $constant = \get_class($object).'::'.$constant;
+            $constant = $object::class.'::'.$constant;
         }
 
         if (!\defined($constant)) {
@@ -1667,7 +1694,7 @@ final class CoreExtension extends AbstractExtension
             }
 
             if (match (true) {
-                \is_array($object) => \array_key_exists($arrayItem, $object),
+                \is_array($object) => \array_key_exists($arrayItem = (string) $arrayItem, $object),
                 $object instanceof \ArrayAccess => $object->offsetExists($arrayItem),
                 default => false,
             }) {
@@ -1688,9 +1715,13 @@ final class CoreExtension extends AbstractExtension
                 }
 
                 if ($object instanceof \ArrayAccess) {
-                    $message = \sprintf('Key "%s" in object with ArrayAccess of class "%s" does not exist.', $arrayItem, \get_class($object));
+                    if (\is_object($arrayItem) || \is_array($arrayItem)) {
+                        $message = \sprintf('Key of type "%s" does not exist in ArrayAccess-able object of class "%s".', get_debug_type($arrayItem), get_debug_type($object));
+                    } else {
+                        $message = \sprintf('Key "%s" does not exist in ArrayAccess-able object of class "%s".', $arrayItem, get_debug_type($object));
+                    }
                 } elseif (\is_object($object)) {
-                    $message = \sprintf('Impossible to access a key "%s" on an object of class "%s" that does not implement ArrayAccess interface.', $item, \get_class($object));
+                    $message = \sprintf('Impossible to access a key "%s" on an object of class "%s" that does not implement ArrayAccess interface.', $item, get_debug_type($object));
                 } elseif (\is_array($object)) {
                     if (!$object) {
                         $message = \sprintf('Key "%s" does not exist as the sequence/mapping is empty.', $arrayItem);
@@ -1786,7 +1817,7 @@ final class CoreExtension extends AbstractExtension
 
         static $cache = [];
 
-        $class = \get_class($object);
+        $class = $object::class;
 
         // object method
         // precedence: getXxx() > isXxx() > hasXxx()
@@ -1811,7 +1842,7 @@ final class CoreExtension extends AbstractExtension
                 } elseif ('h' === $lcName[0] && str_starts_with($lcName, 'has')) {
                     $name = substr($method, 3);
                     $lcName = substr($lcName, 3);
-                    if (\in_array('is'.$lcName, $lcMethods)) {
+                    if (\in_array('is'.$lcName, $lcMethods, true)) {
                         continue;
                     }
                 } else {
